# comic-get.pl - Utility for fetching comics from the internet
# Copyright (C) 2004-2006 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package ComicGet::Index;

use strict;
use warnings FATAL => 'all';

use ComicGet::Config;

my @cached_index;
my $cached_index_read = 0;


# Line format:
# id <tab> uri
#
# Known hash keys:
#   id
#   uri


=item line_to_hash
Split database line into info hash.

0: Line
r: hash
=cut
sub line_to_hash($) {
	my $line = shift;
	my @info = split(/\t/, $line);

	my %t = (
		'id' => $info[0],
		'uri' => $info[1]
	);
	return %t;
}


=item hash_to_line
Write hash into database line.

0: Hash
r: Line
=cut
sub hash_to_line(\%) {
	my %t = %{(shift)};
	my $line = "" .
		$t{'id'} . "\t" .
		$t{'uri'};
	return $line;
}


=item get_index
Read index.

r: Array of hash
=cut
sub get_index() {
	return @cached_index if ($cached_index_read == 1);

	@cached_index = ();
	my $file = &ComicGet::Files::index();
	my $data = &ComicGet::Common::read_file($file);
	if (defined($data)) {
		push(@cached_index, split(/\n/, $data));
	} else {
		if (-e $file) {
			print "E: Cannot read \"$file\": " .
				"$ComicGet::Common::error\n";
			print "Bailing out...\n";
			exit(1);
		}
		print "I: No index, assuming first time use.\n";
		if (! &ComicGet::Common::touch($file)) {
			print "E: Cannot write \"$file\": " .
				"$ComicGet::Common::error\n";
			print "Bailing out...\n";
			exit(1);
		}
	}

	$cached_index_read = 1;
	return @cached_index;
}


=item save
Save index.

0: New index.
=cut
sub save() {
	my @new_index = @_;
	my $file = &ComicGet::Files::index();

	&ComicGet::Common::save_file($file, join("\n", @new_index))
			or die("Cannot write \"$file\": " .
				$ComicGet::Common::error);
	@cached_index = @new_index;
}


=item insert
Insert index entry. If ID already exists, old entry is overwritten.

0: Repository ID
1: Repository URI
=cut
sub insert($$) {
	my ($id, $uri) = @_;
	my %line = (
		'id' => $id,
		'uri' => $uri
	);

	&get_index();

	my @new_index;
	foreach my $line (&get_index()) {
		my %info = &line_to_hash($line);
		push(@new_index, $line) if ($info{'id'} ne $id);
	}
	push(@new_index, &hash_to_line(\%line));

	&save(@new_index);
}


=item source_by_id
Get repository URI by given identifier.

0: Repository identifier
r: URI
=cut
sub source_by_id($) {
	my $id = shift;
	foreach my $line (&get_index()) {
		my %info = &line_to_hash($line);
		return $info{'uri'} if ($info{'id'} eq $id);
	}
	return undef;
}


=item id_by_source
Get repository identifier by given URI.

0: Repository URI
r: Identifier
=cut
sub id_by_source($) {
	my $uri = shift;
	foreach my $line (&get_index()) {
		my %info = &line_to_hash($line);
		return $info{'id'} if ($info{'uri'} eq $uri);
	}
	return undef;
}


1;
